//=============================================================================================================
/**
* @file     cluststcworker.h
* @author   Christoph Dinh <chdinh@nmr.mgh.harvard.edu>;
*           Matti Hamalainen <msh@nmr.mgh.harvard.edu>
* @version  1.0
* @date     June, 2014
*
* @section  LICENSE
*
* Copyright (C) 2014, Christoph Dinh and Matti Hamalainen. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that
* the following conditions are met:
*     * Redistributions of source code must retain the above copyright notice, this list of conditions and the
*       following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
*       the following disclaimer in the documentation and/or other materials provided with the distribution.
*     * Neither the name of MNE-CPP authors nor the names of its contributors may be used
*       to endorse or promote products derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*
*
* @brief    ClustStcWorker class declaration
*
*/

#ifndef CLUSTSTCWORKER_H
#define CLUSTSTCWORKER_H

//*************************************************************************************************************
//=============================================================================================================
// QT INCLUDES
//=============================================================================================================

#include <QObject>
#include <QList>
#include <QThread>
#include <QSharedPointer>
#include <QMutex>


//*************************************************************************************************************
//=============================================================================================================
// Eigen INCLUDES
//=============================================================================================================

#include <Eigen/Core>


//*************************************************************************************************************
//=============================================================================================================
// DEFINE NAMESPACE DISP3DLIB
//=============================================================================================================

namespace DISP3DLIB
{

//*************************************************************************************************************
//=============================================================================================================
// USED NAMESPACES
//=============================================================================================================

using namespace Eigen;

//=============================================================================================================
/**
* Worker which schedules data with the right timing
*
* @brief Data scheduler
*/
class ClustStcWorker : public QThread
{
    Q_OBJECT
public:
    typedef QSharedPointer<ClustStcWorker> SPtr;            /**< Shared pointer type for StcWorker class. */
    typedef QSharedPointer<const ClustStcWorker> ConstSPtr; /**< Const shared pointer type for StcWorker class. */

    ClustStcWorker(QObject *parent = 0);

    ~ClustStcWorker();

//    void setIntervall(int intervall);

    void addData(QList<VectorXd> &data);

    void clear();

    void setAverage(qint32 samples);

    void setInterval(int usec);

    void setLoop(bool looping);

    void stop();

signals:
    void stcSample(Eigen::VectorXd sample);

protected:
    virtual void run();

private:
    QMutex m_qMutex;
    QList<VectorXd> m_data;   /**< List that holds the fiff matrix data <n_channels x n_samples> */

    bool m_bIsRunning;
    bool m_bIsLooping;

    qint32 m_iAverageSamples;
    qint32 m_iCurrentSample;
    qint32 m_iUSecIntervall;
};

} // NAMESPACE

#endif // CLUSTSTCWORKER_H
