//=============================================================================================================
/**
* @file     babymeghpidlg.h
* @author   Limin Sun <liminsun@nmr.mgh.harvard.edu>
*           Matti Hamalainen <msh@nmr.mgh.harvard.edu>
* @version  1.0
* @date     March, 2015
*
* @section  LICENSE
*
* Copyright (C) 2015, Limin Sun and Matti Hamalainen. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that
* the following conditions are met:
*     * Redistributions of source code must retain the above copyright notice, this list of conditions and the
*       following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
*       the following disclaimer in the documentation and/or other materials provided with the distribution.
*     * Neither the name of MNE-CPP authors nor the names of its contributors may be used
*       to endorse or promote products derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*
*
* @brief    Contains the declaration of the BabyMEGHPI class.
*
*/


#ifndef BABYMEGHPIDGL_H
#define BABYMEGHPIDGL_H

#include <QDialog>
#include <QAbstractButton>

//*************************************************************************************************************
//=============================================================================================================
// FIFF INCLUDES
//=============================================================================================================

#include <fiff/fiff_info.h>
#include <fiff/fiff.h>


#include "../babymeg.h"


using namespace FIFFLIB;


namespace Ui {
class babymeghpidgl;
}
namespace BabyMEGPlugin
{


//*************************************************************************************************************
//=============================================================================================================
// FORWARD DECLARATIONS
//=============================================================================================================

class BabyMEG;
class babymeghpidgl : public QDialog
{
    Q_OBJECT

public:
    explicit babymeghpidgl(BabyMEG* p_pBabyMEG,QWidget *parent = 0);
    ~babymeghpidgl();
    //=========================================================================================================
    /**
    * Read Polhemus data from fif file
    *
    */
    void ReadPolhemusDig(QString fileName);

protected:
     virtual void closeEvent( QCloseEvent * event );

public slots:
    //=========================================================================================================
    /**
    * Load a Polhemus file name
    *
    */
    void bnLoadPolhemusFile();
    void OKProc(QAbstractButton *b);
    void CancelProc();

signals:
    void SendHPIFiffInfo(FiffInfo);

private:
    Ui::babymeghpidgl *ui;
public:
      BabyMEG*   m_pBabyMEG;
      FiffInfo info;
      QString FileName_HPI;
};
} //NAMESPACE
#endif // BABYMEGHPIDGL_H
